static char version[] =
"hcal version 1.0 - postscript hebrew calendar generator\n\n\
copyright 1994,1995 - Steven Weintraub";

/*
 
    hcal version 1.0 - postscript hebrew calendar generator
    copyright 1994,1995 - Steven Weintraub
 
    This program is modified and distribed by Steven Weintraub.	 It
    can be freely distributed without permission as long as this
    head is retained in all files it appears and the LICENCE file
    accompanies it.
 
*/

/*
 *
 *   SUN.C   Version 1.0  Michael Schwartz  December 25, 1984
 *
 * This program uses the USNO algorithm to calculate sunrise and sunset
 * times from standard lattitudes and longitudes.  It has been set up as
 * system independently as possible, with the exception being a function
 * which gets the current (default) date and day of the year.  It is meant
 * to be particularly useful for Jewish persons in out-of-the-way locations
 * who might need to know the time of candle_lighting or havdalah.  With
 * little difficulty, z'man k'riath shema can be calculated and added to the
 * routines.
 *
 * Inquiries can reach me at PO BOX 24536, Denver, Colorado 80224
 * (a place no longer 'out of the way')
 *
 * Ref:	Almanac for Computers, pub. by US NAVAL OBSERVATORY
 * Usage:
 * sun [-c] [-l[lat]] [-L[long]] [-M] [-m[1-12|a]] [-d[day]]
 *     [-t<S|C|N|A>]  [-<12|24>]
 *  Latitude in deg,min'sec" N;	Longitude in deg,min'sec" W
 *
 * Accuracy:  The USNO claims accuracy withing 2 minutes except at extreme
 *  northern or southern lattitudes.  Comparison to local NWS charts for
 *  sunrise and sunset (which are cheap and easy to come by) shows that with
 *  the double precision calculations, the charts produced by this program
 *  are no more than 1 minute removed from those charts in lattitudes lower
 *  than 41 degrees.  Candle lighting times agree with those on popular
 *  calendars also to the 1 minute accuracy.
 *
 *  The program, unlike its fortran predecessor has a number of
 * important options and defaults.  It is capable of getting today's
 * sunrise and sunset at a default location (now Denver, of course),
 * producing a calendar-like table of a month or a year, and allowing the
 * user to produce reams of data without reinvoking the program.
 *
 * Permission is granted to distribute and use this program as desired.
 *
 * Bugs:  Automatic calculation of non-default time zones will not produce
 *  correct local times	(using the APPROXIMATE_TZ define).  Otherwise,
 *  when lattitudes and longitudes are specified, the result is in Zulu	(UT).
 */

#include "hcal.h"

/*
 * This program will calculate sunset times for Jerusalem (35 14 E / 31 46 N)
 *
 * lambda = longitude
 * phi = latitude
 *
 * For 1978, M = .985600t -3.251 -- M is the Sun's mean anomaly
 * L = M + 1.916 SIN(M)	+ .020SIN(2M) +	282.565	-- L is Sun's TRUE long.
 * TANa	= .91746 TAN L		-- a is Sun's right ascension
 * SIN DELTA = .39782 SIN L		-- Sun's declination
 * COS h = (COSz - SIN DELTA*SIN PHI)/(COS DELTA*COS PHI)
 * T = h + a - 0.065710t -6.620	 -- h is local hour angle, a now in hours
 * UT =	T + LAMBDA --UT is GMT.	 LAMBDA is now in hours	(deg / 15)
 *
 * Where Z is the zenith distance at phenomena in question. Samples:
 * 90,50'  regular sunrise/set	(atmospheric refraction	+ disk diameter)
 * 96	   civil twilight
 * 102	   nautical twilight
 * 106	   astronomical twilight
 */

#define PI  3.14159265
#define TODEC(a, b, c) ((double)a + (double)b/60.0 + (double)c/3600.0)
#define DEGRAD (PI/180.0)
#define RADDEG (180.0/PI)
#define D 0.91746
#define E 0.39782
#define M(x) (0.985600*x - 3.251)
#define L(x) (x	+ 1.916*sin(DEGRAD*x) +	0.020*sin(2*DEGRAD*x) +	282.565)
#define ADJ(x) (-0.065710*x - 6.620)

/*
 *		  --- This is where the REAL work is done ---
 * This routine will seem FORTRANy.  An attempt has been made to
 * mirror the procedure outlined in the USGS publication; very little
 * optimization has been done, so that the connection between the
 * publication and the software will be extremely clear.
 * Thus, all angles are preserved in degrees (degrees are converted to
 * hours by the observation that 360 degrees = 24 hours	-- so 15.0
 * degrees are 1 hour).
 */
#if defined(PROTOTYPES)
suntime(int day,int is_rise)
#else
suntime(day,is_rise) int day; int is_rise;
#endif
{
	double sunset;
	double lo_hr, lambda, phi;
	double cos_phi,	sin_phi, cos_z;
	double sin(), cos(), acos(), tan(), atan(), sqrt();
	double fabs();
	double t, xm, xl, a, a_hr, sin_del, cos_del, h,	h_hr;
	int hour,min;
 
	cos_z =	cos(DEGRAD * TODEC(90, 50, 0));

	lambda = TODEC(city->longdeg, city->longmin, city->longsec);
	lo_hr =	lambda / 15.0;
	phi = TODEC(city->latdeg, city->latmin,	city->latsec);
	cos_phi	= cos(DEGRAD * phi);
	sin_phi	= sin(DEGRAD * phi);


	t = (double) day;
	if (is_rise)
		t += (18.0 + lo_hr) / 24.0;
	else
		t += (6.0 + lo_hr) / 24.0;

	xm = M(t);
	xl = L(xm);
	a = RADDEG * atan(D * tan(DEGRAD * xl));

	if (fabs(a + 360.0 - xl) > 90.0)
		a += 180.0;
	if (a >	360.0)
		a -= 360.0;

	a_hr = a / 15.0;
	sin_del	= E * sin(DEGRAD * xl);
	cos_del	= sqrt(1.0 - sin_del * sin_del); /* cos del must ALWAYS be >0 */
	h = (cos_z - sin_del * sin_phi)	/ (cos_del * cos_phi);

	if (fabs(h) > 1.0)
		return -1;

	h = RADDEG * acos(h);

	if (is_rise)
		h = 360.0 - h;	/* Puts sunrise in correct quadrant */
	h_hr = h / 15.0;
	sunset = h_hr +	a_hr + ADJ(t) +	lo_hr + 12 + city->tzone;
	if (is_dst(day,current_year))
		sunset++;
	if (sunset < 0)
		sunset += 24;
	hour = sunset;
	min = (int) ((sunset - (double)	hour) *	60.0 + 0.5);
	if (min	>= 60) {
		hour +=	1;
		min  -=	60;
	}
	if (hour > 24) {
		hour -=	24;
	}
	if (is_rise && hour > 12)
		hour -= 12;
	if (!is_rise && hour < 12)
		hour += 12;

	return hour * 60 + min;
}
